package com.bergerkiller.bukkit.nolagg;

import java.util.Set;
import java.util.logging.Level;

import net.minecraft.server.Chunk;
import net.minecraft.server.WorldServer;

@SuppressWarnings("rawtypes")
public class ChunkOperation implements Comparable {
	private int mode = 0;
	public final Chunk c;
	public Set ticks;
	public ChunkOperation(final Chunk chunk, Type type) {
		this.c = chunk;
		switch(type) {
		case LIGHTING : this.mode = 1; break;
		case UNLOAD : this.mode = 2; break;
		case AUTOSAVE : this.mode = 3; break;
		}
	}
	
	public static enum Type{LIGHTING, UNLOAD, AUTOSAVE};
	
	public int compareTo(Object object) {
		if (object instanceof ChunkOperation) {
			return ((ChunkOperation) object).mode - this.mode;
		} else {
			return 0;
		}
	}
	
	public int getMode() {
		return this.mode;
	}
		
	/*
	 * Whatever needs to happen when this object is being scheduled (synced)
	 */
	public void preexecute() {
		if (this.mode == 1) {
			//remove queued chunks for this chunk
		    ChunkSendQueue.deQueueGlobal(this.c);
		} else if (this.mode == 2 || this.mode == 3) {
			this.ticks = ChunkSaving.getTickList(this.getWorld());
		}
	}
		
	public WorldServer getWorld() {
		return (WorldServer) this.c.world;
	}
		
	/*
	 * What needs to happen when the scheduler finds this object
	 */
	public void execute() {
		//1 = fix lighting
		//2 = save - unload mode
		//3 = save - autosave mode
		//save
		if (this.mode == 1) {
			c.h();
			c.initLighting();
			boolean haserror = true;
			int loops = -1;
			while (haserror) {
				if (loops > 20) {
					NoLagg.log(Level.WARNING, "Failed to fix all lighting issues in chunk [" + c.x + "/" + c.z + "/" + c.world.getWorld().getName() + "]");
					break;
				}
				haserror = false;
				loops++;
				for (int x = 0; x < 16; x++) {
					for (int z = 0; z < 16; z++) {
						for (int y = 127; y > 0; y--) {
							if (c.getTypeId(x, y, z) == 0) {
								if (c.h.a(x, y, z) == 0) {
									//air is all dark? Better fix this!
									//check if nearby blocks with higher light exist
									int maxlight = 1;
									if (x > 0) maxlight = Math.max(maxlight, c.c(x - 1, y, z, c.world.k));
									if (z > 0) maxlight = Math.max(maxlight, c.c(x, y, z - 1, c.world.k));
									if (y > 0) maxlight = Math.max(maxlight, c.c(x, y - 1, z, c.world.k));
									if (x < 15) maxlight = Math.max(maxlight, c.c(x + 1, y, z, c.world.k));
									if (z < 15) maxlight = Math.max(maxlight, c.c(x, y, z + 1, c.world.k));
									if (y < 127) maxlight = Math.max(maxlight, c.c(x, y + 1, z, c.world.k));
									--maxlight;
									if (maxlight > 0) {
										haserror = true;
										c.h.a(x, y, z, maxlight);
									}
								}
							}
						}
					}
				}
			}
			//queue
			ChunkSendQueue.scheduleQueueGlobal(this.c);
		} else if (this.mode == 2) {
			//unload
			ChunkSaving.saveChunk(this.c, this.ticks);
		} else if (this.mode == 3) {
			//autosave - detach entities!
			ChunkSaving.saveChunk(ChunkHandler.cloneChunk(this.c), this.ticks);
		}
	}
	
}
