package com.bergerkiller.bukkit.nolagg;

import java.util.Collection;
import java.util.List;

import org.bukkit.Bukkit;
import org.bukkit.block.BlockFace;
import org.bukkit.craftbukkit.CraftServer;
import org.bukkit.craftbukkit.CraftWorld;
import org.bukkit.craftbukkit.entity.CraftEntity;
import org.bukkit.entity.ExperienceOrb;
import org.bukkit.entity.Item;
import org.bukkit.event.Cancellable;
import org.bukkit.event.Event;

import net.minecraft.server.Chunk;
import net.minecraft.server.Entity;
import net.minecraft.server.EntityPlayer;
import net.minecraft.server.Packet;
import net.minecraft.server.World;
import net.minecraft.server.WorldServer;

public class Util {
	public static final int view = Bukkit.getServer().getViewDistance();
	public static final int chunkarea = (view + view + 1) * (view + view + 1);
	public static final int blockView = 32 + (Bukkit.getServer().getViewDistance() << 4);
		
	public static void broadcast(Chunk chunk, Packet packet) {
		broadcast(chunk.world, chunk.x, chunk.z, packet);
	}
	public static void broadcast(Chunk chunk, Collection<Packet> packets) {
		broadcast(chunk.world, chunk.x, chunk.z, packets);
	}
	public static void broadcast(Chunk chunk, Packet[] packets) {
		broadcast(chunk.world, chunk.x, chunk.z, packets);
	}
	public static void broadcast(World world, int cx, int cz, Packet packet) {
		for (Object player : world.players) {
			if (player instanceof EntityPlayer) {
				EntityPlayer ep = (EntityPlayer) player;
				if (isNear(ep, cx, cz)) {
					ep.netServerHandler.sendPacket(packet);
				}
			}
		}
	}
	public static void broadcast(World world, int cx, int cz, Collection<Packet> packets) {
		for (Object player : world.players) {
			if (player instanceof EntityPlayer) {
				EntityPlayer ep = (EntityPlayer) player;
				if (isNear(ep, cx, cz)) {
					for (Packet p : packets) {
						ep.netServerHandler.sendPacket(p);
					}
				}
			}
		}
	}
	public static void broadcast(World world, int cx, int cz, Packet[] packets) {
		for (Object player : world.players) {
			if (player instanceof EntityPlayer) {
				EntityPlayer ep = (EntityPlayer) player;
				if (isNear(ep, cx, cz)) {
					for (Packet p : packets) {
						ep.netServerHandler.sendPacket(p);
					}
				}
			}
		}
	}
	public static boolean isNear(Entity e, int cx, int cz) {
		return isNear((int) e.locX >> 4, (int) e.locZ >> 4, cx, cz);
	}
	public static boolean isNear(int cx1, int cz1, int cx2, int cz2) {
		cx1 -= cx2;
		if (cx1 > 15) return false;
		if (cx1 < -15) return false;
		cz1 -= cz2;
		if (cz1 > 15) return false;
		if (cz1 < -15) return false;
		return true;
	}
	
	public static List<WorldServer> getWorlds() {
		return ((CraftServer) Bukkit.getServer()).getHandle().server.worlds;
	}
    public static long toLong(int msw, int lsw) {
        return ((long) msw << 32) + lsw - Integer.MIN_VALUE;
    }
    public static boolean callEvent(Event event) {
    	Bukkit.getServer().getPluginManager().callEvent(event);
    	if (event instanceof Cancellable) {
    	    return !((Cancellable) event).isCancelled();
    	} else {
    		return true;
    	}
    }
	public static BlockFace getDirection(int yaw) {
        while (yaw <= -180) yaw += 360;
        while (yaw > 180) yaw -= 360;
		switch ((int) yaw) {
		case 0 : return BlockFace.NORTH;
		case 45 : return BlockFace.NORTH_EAST;
		case 90 : return BlockFace.EAST;
		case 135 : return BlockFace.SOUTH_EAST;
		case 180 : return BlockFace.SOUTH;
		case -135 : return BlockFace.SOUTH_WEST;
		case -90 : return BlockFace.WEST;
		case -45 : return BlockFace.NORTH_WEST;
		}
		//Let's apply angle differences
		if (yaw >= -22.5 && yaw < 22.5) {
			return BlockFace.NORTH;
		} else if (yaw >= 22.5 && yaw < 67.5) {
			return BlockFace.NORTH_EAST;
		} else if (yaw >= 67.5 && yaw < 112.5) {
			return BlockFace.EAST;
		} else if (yaw >= 112.5 && yaw < 157.5) {
			return BlockFace.SOUTH_EAST;
		} else if (yaw >= -67.5 && yaw < -22.5) {
			return BlockFace.NORTH_WEST;
		} else if (yaw >= -112.5 && yaw < -67.5) {
			return BlockFace.WEST;
		} else if (yaw >= -157.5 && yaw < -112.5) {
			return BlockFace.SOUTH_WEST;
		} else {
			return BlockFace.SOUTH;
		}
	}	
	
	public static void fillEntities(org.bukkit.World world, List<org.bukkit.entity.Entity> entitylist, List<Item> itemlist, List<ExperienceOrb> orblist) {
		fillEntities(((CraftWorld) world).getHandle(), entitylist, itemlist, orblist);
	}
	public static void fillEntities(World world, List<org.bukkit.entity.Entity> entitylist, List<Item> itemlist, List<ExperienceOrb> orblist) {
		CraftEntity e;
		for (Object obj : world.entityList) {
			e = (CraftEntity) ((net.minecraft.server.Entity) obj).getBukkitEntity();
			if (e.isDead()) continue;
			if (e instanceof Item) {
				if (ItemHandler.isIgnoredItem(e)) continue;
				if (itemlist != null) itemlist.add((Item) e);
			} else if (e instanceof ExperienceOrb) {
				if (orblist != null) orblist.add((ExperienceOrb) e);
			}
			if (entitylist != null) entitylist.add(e);
		}
	}

    public static double round(double Rval, int Rpl) {
    	double p = Math.pow(10, Rpl);
    	return Math.round(Rval * p) / p;
    }
}
